Package: TH.data
Title: TH's Data Archive
Date: 2025-01-17
Version: 1.1-3
Authors@R: c(person("Torsten", "Hothorn", role = c("aut", "cre"),
                    email = "Torsten.Hothorn@R-project.org"))
Description: Contains data sets used in other packages Torsten Hothorn
  maintains.
Depends: R (>= 3.5.0), survival, MASS
Suggests: trtf, tram, rms, coin, ATR, multcomp, gridExtra, vcd,
        colorspace, lattice, knitr
LazyData: yes
VignetteBuilder: knitr
License: GPL-3
NeedsCompilation: no
Packaged: 2025-01-17 14:47:14 UTC; hothorn
Author: Torsten Hothorn [aut, cre]
Maintainer: Torsten Hothorn <Torsten.Hothorn@R-project.org>
Repository: CRAN
Date/Publication: 2025-01-17 18:20:02 UTC
Built: R 4.4.0; ; 2026-07-18 04:53:20 UTC; unix
