Package: TH.data
Title: TH's Data Archive
Date: 2025-11-17
Version: 1.1-5
Authors@R: c(person("Torsten", "Hothorn", role = c("aut", "cre"),
                    email = "Torsten.Hothorn@R-project.org"))
Description: Contains data sets used in other packages Torsten Hothorn
  maintains.
Depends: R (>= 3.5.0), survival, MASS
Suggests: trtf, tram, rms, coin, ATR, multcomp, gridExtra, vcd,
        colorspace, lattice, knitr, dplyr, openxlsx, plyr
LazyData: yes
VignetteBuilder: knitr
License: GPL-3
NeedsCompilation: no
Packaged: 2025-11-17 15:44:06 UTC; hothorn
Author: Torsten Hothorn [aut, cre]
Maintainer: Torsten Hothorn <Torsten.Hothorn@R-project.org>
Repository: CRAN
Date/Publication: 2025-11-17 16:50:08 UTC
